@@ -244,7 +244,7 @@ construct_cal_factory_path (void)
 	static volatile gint counter = 1;
 
 	return g_strdup_printf (
-		"/org/gnome/evolution/dataserver/calendar/%d/%u",
+		"/org/gnome/evolution/dataserver/Calendar/%d/%u",
 		getpid (), g_atomic_int_exchange_and_add (&counter, 1));
 }
 
@@ -522,6 +522,8 @@ remove_data_cal_cb (gpointer data_cl,
 static void
 e_data_cal_factory_init (EDataCalFactory *factory)
 {
+	GError *error = NULL;
+
 	factory->priv = E_DATA_CAL_FACTORY_GET_PRIVATE (factory);
 
 	factory->priv->gdbus_object = e_gdbus_cal_factory_stub_new ();
@@ -549,10 +551,21 @@ e_data_cal_factory_init (EDataCalFactory *factory)
 		(GDestroyNotify) g_free,
 		(GDestroyNotify) NULL);
 
-	e_data_server_module_init ();
+	if (!e_data_server_module_init (BACKENDDIR, &error))
+		g_error ("%s", error->message);
+
 	e_data_cal_factory_register_backends (factory);
 }
 
+static void
+unref_backend_cb (gpointer key, gpointer value, gpointer user_data)
+{
+	ECalBackend *backend = value;
+
+	if (backend)
+		g_object_unref (backend);
+}
+
 static void
 e_data_cal_factory_finalize (GObject *object)
 {
@@ -562,6 +575,8 @@ e_data_cal_factory_finalize (GObject *object)
 
 	g_object_unref (factory->priv->gdbus_object);
 
+	g_hash_table_foreach (factory->priv->backends, unref_backend_cb, NULL);
+
 	g_hash_table_destroy (factory->priv->methods);
 	g_hash_table_destroy (factory->priv->backends);
 	g_hash_table_destroy (factory->priv->calendars);
@@ -807,9 +822,6 @@ offline_state_changed_cb (EOfflineListener *eol,
 		factory, state == EOL_STATE_ONLINE ? Remote : Local);
 }
 
-#define E_DATA_CAL_FACTORY_SERVICE_NAME \
-	"org.gnome.evolution.dataserver.Calendar"
-
 static void
 on_bus_acquired (GDBusConnection *connection,
                  const gchar     *name,
@@ -822,11 +834,11 @@ on_bus_acquired (GDBusConnection *connection,
 	registration_id = e_data_cal_factory_register_gdbus_object (
 		factory,
 		connection,
-		"/org/gnome/evolution/dataserver/calendar/CalFactory",
+		"/org/gnome/evolution/dataserver/CalendarFactory",
 		&error);
 
 	if (error)
-		die ("Failed to register a CalFactory object", error);
+		die ("Failed to register a CalendarFactory object", error);
 
 	g_assert (registration_id > 0);
 }
@@ -862,6 +874,30 @@ on_name_lost (GDBusConnection *connection,
 	}
 }
 
+#ifndef G_OS_WIN32
+static void
+quit_signal (gint sig)
+{
+	g_return_if_fail (sig == SIGQUIT);
+
+	g_print ("Received quit signal...\n");
+	g_main_loop_quit (loop);
+}
+
+static void
+setup_quit_signal (void)
+{
+	struct sigaction sa, osa;
+
+	sigaction (SIGQUIT, NULL, &osa);
+
+	sa.sa_flags = 0;
+	sigemptyset (&sa.sa_mask);
+	sa.sa_handler = quit_signal;
+	sigaction (SIGQUIT, &sa, NULL);
+}
+#endif
+
 gint
 main (gint argc, gchar **argv)
 {
@@ -914,7 +950,7 @@ main (gint argc, gchar **argv)
 		G_CALLBACK (offline_state_changed_cb), factory);
 
 	owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
-		E_DATA_CAL_FACTORY_SERVICE_NAME,
+		CALENDAR_DBUS_SERVICE_NAME,
 		G_BUS_NAME_OWNER_FLAGS_NONE,
 		on_bus_acquired,
 		on_name_acquired,
@@ -925,7 +961,11 @@ main (gint argc, gchar **argv)
 	/* Migrate user data from ~/.evolution to XDG base directories. */
 	e_data_cal_migrate ();
 
-	printf ("Server is up and running...\n");
+	#ifndef G_OS_WIN32
+	setup_quit_signal ();
+	#endif
+
+	g_print ("Server is up and running...\n");
 
 	g_main_loop_run (loop);
 
@@ -933,7 +973,7 @@ main (gint argc, gchar **argv)
 	g_object_unref (eol);
 	g_object_unref (factory);
 
-	printf ("Bye.\n");
+	g_print ("Bye.\n");
 
 	return 0;
 }
