@@ -20,60 +20,67 @@
  *
  */
 
+/* If building without Kerberos support, this class is an empty shell. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <errno.h>
 
-#ifdef HAVE_KRB5
-#include <netdb.h>
 #include <string.h>
-#include <sys/socket.h>
 #include <sys/types.h>
-#ifdef HAVE_HEIMDAL_KRB5
-#include <krb5.h>
-#else
-#include <krb5/krb5.h>
-#endif
-#ifdef HAVE_ET_COM_ERR_H
-#include <et/com_err.h>
-#else
-#ifdef HAVE_COM_ERR_H
-#include <com_err.h>
-#endif
-#endif
-#ifdef HAVE_MIT_KRB5
-#include <gssapi/gssapi.h>
-#include <gssapi/gssapi_generic.h>
-#endif
-#ifdef HAVE_HEIMDAL_KRB5
-#include <gssapi.h>
-#else
-#ifdef  HAVE_SUN_KRB5
-#include <gssapi/gssapi.h>
-#include <gssapi/gssapi_ext.h>
-extern gss_OID gss_nt_service_name;
-#endif
-#endif
-
-#ifndef GSS_C_OID_KRBV5_DES
-#define GSS_C_OID_KRBV5_DES GSS_C_NO_OID
-#endif
 
+#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib-lowlevel.h>
-
 #include "camel-net-utils.h"
 #include "camel-sasl-gssapi.h"
 #include "camel-session.h"
 
+#ifdef HAVE_KRB5
+
+#ifdef HAVE_HEIMDAL_KRB5
+#include <krb5.h>
+#else
+#include <krb5/krb5.h>
+#endif /* HAVE_HEIMDAL_KRB5 */
+
+#ifdef HAVE_ET_COM_ERR_H
+#include <et/com_err.h>
+#else
+#ifdef HAVE_COM_ERR_H
+#include <com_err.h>
+#endif /* HAVE_COM_ERR_H */
+#endif /* HAVE_ET_COM_ERR_H */
+
+#ifdef HAVE_MIT_KRB5
+#include <gssapi/gssapi.h>
+#include <gssapi/gssapi_generic.h>
+#endif /* HAVE_MIT_KRB5 */
+
+#ifdef HAVE_HEIMDAL_KRB5
+#include <gssapi.h>
+#else
+#ifdef HAVE_SUN_KRB5
+#include <gssapi/gssapi.h>
+#include <gssapi/gssapi_ext.h>
+extern gss_OID gss_nt_service_name;
+#endif /* HAVE_SUN_KRB5 */
+#endif /* HAVE_HEIMDAL_KRB5 */
+
+#ifndef GSS_C_OID_KRBV5_DES
+#define GSS_C_OID_KRBV5_DES GSS_C_NO_OID
+#endif
+
 #define DBUS_PATH		"/org/gnome/KrbAuthDialog"
 #define DBUS_INTERFACE		"org.gnome.KrbAuthDialog"
 #define DBUS_METHOD		"org.gnome.KrbAuthDialog.acquireTgt"
 
+#define CAMEL_SASL_GSSAPI_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), CAMEL_TYPE_SASL_GSSAPI, CamelSaslGssapiPrivate))
+
 CamelServiceAuthType camel_sasl_gssapi_authtype = {
 	N_("GSSAPI"),
 
@@ -103,69 +110,16 @@ struct _CamelSaslGssapiPrivate {
 	gss_name_t target;
 };
 
-static GByteArray *gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex);
+#endif /* HAVE_KRB5 */
 
-static CamelSaslClass *parent_class = NULL;
+G_DEFINE_TYPE (CamelSaslGssapi, camel_sasl_gssapi, CAMEL_TYPE_SASL)
+
+#ifdef HAVE_KRB5
 
 static void
-camel_sasl_gssapi_class_init (CamelSaslGssapiClass *klass)
-{
-	CamelSaslClass *camel_sasl_class = CAMEL_SASL_CLASS (klass);
-
-	parent_class = CAMEL_SASL_CLASS (camel_type_get_global_classfuncs (camel_sasl_get_type ()));
-
-	/* virtual method overload */
-	camel_sasl_class->challenge = gssapi_challenge;
-}
-
-static void
-camel_sasl_gssapi_init (gpointer object, gpointer klass)
-{
-	CamelSaslGssapi *gssapi = CAMEL_SASL_GSSAPI (object);
-
-	gssapi->priv = g_new (struct _CamelSaslGssapiPrivate, 1);
-	gssapi->priv->state = GSSAPI_STATE_INIT;
-	gssapi->priv->ctx = GSS_C_NO_CONTEXT;
-	gssapi->priv->target = GSS_C_NO_NAME;
-}
-
-static void
-camel_sasl_gssapi_finalize (CamelObject *object)
-{
-	CamelSaslGssapi *gssapi = CAMEL_SASL_GSSAPI (object);
-	guint32 status;
-
-	if (gssapi->priv->ctx != GSS_C_NO_CONTEXT)
-		gss_delete_sec_context (&status, &gssapi->priv->ctx, GSS_C_NO_BUFFER);
-
-	if (gssapi->priv->target != GSS_C_NO_NAME)
-		gss_release_name (&status, &gssapi->priv->target);
-
-	g_free (gssapi->priv);
-}
-
-CamelType
-camel_sasl_gssapi_get_type (void)
-{
-	static CamelType type = CAMEL_INVALID_TYPE;
-
-	if (type == CAMEL_INVALID_TYPE) {
-		type = camel_type_register (
-			camel_sasl_get_type (),
-			"CamelSaslGssapi",
-			sizeof (CamelSaslGssapi),
-			sizeof (CamelSaslGssapiClass),
-			(CamelObjectClassInitFunc) camel_sasl_gssapi_class_init,
-			NULL,
-			(CamelObjectInitFunc) camel_sasl_gssapi_init,
-			(CamelObjectFinalizeFunc) camel_sasl_gssapi_finalize);
-	}
-
-	return type;
-}
-
-static void
-gssapi_set_exception (OM_uint32 major, OM_uint32 minor, CamelException *ex)
+gssapi_set_exception (OM_uint32 major,
+                      OM_uint32 minor,
+                      GError **error)
 {
 	const gchar *str;
 
@@ -215,7 +169,27 @@ gssapi_set_exception (OM_uint32 major, OM_uint32 minor, CamelException *ex)
 		str = _("Bad authentication response from server.");
 	}
 
-	camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE, str);
+	g_set_error (
+		error, CAMEL_SERVICE_ERROR,
+		CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+		"%s", str);
+}
+
+static void
+sasl_gssapi_finalize (GObject *object)
+{
+	CamelSaslGssapi *sasl = CAMEL_SASL_GSSAPI (object);
+	guint32 status;
+
+	if (sasl->priv->ctx != GSS_C_NO_CONTEXT)
+		gss_delete_sec_context (
+			&status, &sasl->priv->ctx, GSS_C_NO_BUFFER);
+
+	if (sasl->priv->target != GSS_C_NO_NAME)
+		gss_release_name (&status, &sasl->priv->target);
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_sasl_gssapi_parent_class)->finalize (object);
 }
 
 /* DBUS Specific code */
@@ -223,54 +197,52 @@ gssapi_set_exception (OM_uint32 major, OM_uint32 minor, CamelException *ex)
 static gboolean
 send_dbus_message (gchar *name)
 {
-	DBusMessage *message, *reply;
-	DBusError dbus_error;
 	gint success = FALSE;
-	DBusConnection *bus = NULL;
+	GError *error = NULL;
+	GDBusConnection *connection;
+	GDBusMessage *message, *reply;
+
+	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+	if (error) {
+		g_warning ("could not get system bus: %s\n", error->message);
+		g_error_free (error);
 
-	dbus_error_init (&dbus_error);
-	if (!(bus = dbus_bus_get (DBUS_BUS_SESSION, &dbus_error))) {
-		g_warning ("could not get system bus: %s\n", dbus_error.message);
-		dbus_error_free (&dbus_error);
 		return FALSE;
 	}
 
-	dbus_error_free (&dbus_error);
-
-	dbus_connection_setup_with_g_main (bus, NULL);
-	dbus_connection_set_exit_on_disconnect (bus, FALSE);
+	g_dbus_connection_set_exit_on_close (connection, FALSE);
 
 	/* Create a new message on the DBUS_INTERFACE */
-	if (!(message = dbus_message_new_method_call (DBUS_INTERFACE, DBUS_PATH, DBUS_INTERFACE, "acquireTgt"))) {
-		g_object_unref (bus);
+	message = g_dbus_message_new_method_call (DBUS_INTERFACE, DBUS_PATH, DBUS_INTERFACE, "acquireTgt");
+	if (!message) {
+		g_object_unref (connection);
 		return FALSE;
 	}
+
 	/* Appends the data as an argument to the message */
-	if (strchr(name, '\\'))
-		name = strchr(name, '\\');
-	dbus_message_append_args (message,
-				  DBUS_TYPE_STRING, &name,
-				  DBUS_TYPE_INVALID);
-	dbus_error_init(&dbus_error);
+	if (strchr (name, '\\'))
+		name = strchr (name, '\\');
+	g_dbus_message_set_body (message, g_variant_new ("(s)", name));
 
 	/* Sends the message: Have a 300 sec wait timeout  */
-	reply = dbus_connection_send_with_reply_and_block (bus, message, 300 * 1000, &dbus_error);
+	reply = g_dbus_connection_send_message_with_reply_sync (connection, message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, 300 * 1000, NULL, NULL, &error);
 
-	if (dbus_error_is_set(&dbus_error))
-		g_warning ("%s: %s\n", dbus_error.name, dbus_error.message);
-	dbus_error_free(&dbus_error);
+	if (error) {
+		g_warning ("%s: %s\n", G_STRFUNC, error->message);
+		g_error_free (error);
+	}
 
-        if (reply)
-        {
-                dbus_error_init(&dbus_error);
-                dbus_message_get_args(reply, &dbus_error, DBUS_TYPE_BOOLEAN, &success, DBUS_TYPE_INVALID);
-                dbus_error_free(&dbus_error);
-                dbus_message_unref(reply);
-        }
+	if (reply) {
+		GVariant *body = g_dbus_message_get_body (reply);
+
+		success = body && g_variant_get_boolean (body);
+
+		g_object_unref (reply);
+	}
 
 	/* Free the message */
-	dbus_message_unref (message);
-	dbus_connection_unref (bus);
+	g_object_unref (message);
+	g_object_unref (connection);
 
 	return success;
 }
@@ -278,9 +250,13 @@ send_dbus_message (gchar *name)
 /* END DBus stuff */
 
 static GByteArray *
-gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
+sasl_gssapi_challenge_sync (CamelSasl *sasl,
+                            GByteArray *token,
+                            GCancellable *cancellable,
+                            GError **error)
 {
-	struct _CamelSaslGssapiPrivate *priv = CAMEL_SASL_GSSAPI (sasl)->priv;
+	CamelSaslGssapiPrivate *priv;
+	CamelService *service;
 	OM_uint32 major, minor, flags, time;
 	gss_buffer_desc inbuf, outbuf;
 	GByteArray *challenge = NULL;
@@ -290,17 +266,26 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 	gss_OID mech;
 	gchar *str;
 	struct addrinfo *ai, hints;
+	const gchar *service_name;
+
+	priv = CAMEL_SASL_GSSAPI_GET_PRIVATE (sasl);
+
+	service = camel_sasl_get_service (sasl);
+	service_name = camel_sasl_get_service_name (sasl);
 
 	switch (priv->state) {
 	case GSSAPI_STATE_INIT:
-		memset(&hints, 0, sizeof(hints));
+		memset (&hints, 0, sizeof (hints));
 		hints.ai_flags = AI_CANONNAME;
-		ai = camel_getaddrinfo(sasl->service->url->host?sasl->service->url->host:"localhost", NULL, &hints, ex);
+		ai = camel_getaddrinfo (
+			service->url->host ?
+			service->url->host : "localhost",
+			NULL, &hints, cancellable, error);
 		if (ai == NULL)
 			return NULL;
 
-		str = g_strdup_printf("%s@%s", sasl->service_name, ai->ai_canonname);
-		camel_freeaddrinfo(ai);
+		str = g_strdup_printf("%s@%s", service_name, ai->ai_canonname);
+		camel_freeaddrinfo (ai);
 
 		inbuf.value = str;
 		inbuf.length = strlen (str);
@@ -308,7 +293,7 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 		g_free (str);
 
 		if (major != GSS_S_COMPLETE) {
-			gssapi_set_exception (major, minor, ex);
+			gssapi_set_exception (major, minor, error);
 			return NULL;
 		}
 
@@ -318,8 +303,10 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 		break;
 	case GSSAPI_STATE_CONTINUE_NEEDED:
 		if (token == NULL) {
-			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					     _("Bad authentication response from server."));
+			g_set_error (
+				error, CAMEL_SERVICE_ERROR,
+				CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+				_("Bad authentication response from server."));
 			return NULL;
 		}
 
@@ -345,10 +332,10 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 			if (major == (OM_uint32)GSS_S_FAILURE &&
 			    (minor == (OM_uint32)KRB5KRB_AP_ERR_TKT_EXPIRED ||
 			     minor == (OM_uint32)KRB5KDC_ERR_NEVER_VALID) &&
-			    send_dbus_message (sasl->service->url->user))
+			    send_dbus_message (service->url->user))
 					goto challenge;
 
-			gssapi_set_exception (major, minor, ex);
+			gssapi_set_exception (major, minor, error);
 			return NULL;
 		}
 
@@ -360,8 +347,10 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 		break;
 	case GSSAPI_STATE_COMPLETE:
 		if (token == NULL) {
-			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					     _("Bad authentication response from server."));
+			g_set_error (
+				error, CAMEL_SERVICE_ERROR,
+				CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+				_("Bad authentication response from server."));
 			return NULL;
 		}
 
@@ -370,13 +359,15 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 
 		major = gss_unwrap (&minor, priv->ctx, &inbuf, &outbuf, &conf_state, &qop);
 		if (major != GSS_S_COMPLETE) {
-			gssapi_set_exception (major, minor, ex);
+			gssapi_set_exception (major, minor, error);
 			return NULL;
 		}
 
 		if (outbuf.length < 4) {
-			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					     _("Bad authentication response from server."));
+			g_set_error (
+				error, CAMEL_SERVICE_ERROR,
+				CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+				_("Bad authentication response from server."));
 #ifndef HAVE_HEIMDAL_KRB5
 			gss_release_buffer (&minor, &outbuf);
 #endif
@@ -385,19 +376,21 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 
 		/* check that our desired security layer is supported */
 		if ((((guchar *) outbuf.value)[0] & DESIRED_SECURITY_LAYER) != DESIRED_SECURITY_LAYER) {
-			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					     _("Unsupported security layer."));
+			g_set_error (
+				error, CAMEL_SERVICE_ERROR,
+				CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+				_("Unsupported security layer."));
 #ifndef HAVE_HEIMDAL_KRB5
 			gss_release_buffer (&minor, &outbuf);
 #endif
 			return NULL;
 		}
 
-		inbuf.length = 4 + strlen (sasl->service->url->user);
+		inbuf.length = 4 + strlen (service->url->user);
 		inbuf.value = str = g_malloc (inbuf.length);
 		memcpy (inbuf.value, outbuf.value, 4);
 		str[0] = DESIRED_SECURITY_LAYER;
-		memcpy (str + 4, sasl->service->url->user, inbuf.length - 4);
+		memcpy (str + 4, service->url->user, inbuf.length - 4);
 
 #ifndef HAVE_HEIMDAL_KRB5
 		gss_release_buffer (&minor, &outbuf);
@@ -405,7 +398,7 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 
 		major = gss_wrap (&minor, priv->ctx, FALSE, qop, &inbuf, &conf_state, &outbuf);
 		if (major != GSS_S_COMPLETE) {
-			gssapi_set_exception (major, minor, ex);
+			gssapi_set_exception (major, minor, error);
 			g_free (str);
 			return NULL;
 		}
@@ -420,7 +413,7 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 
 		priv->state = GSSAPI_STATE_AUTHENTICATED;
 
-		sasl->authenticated = TRUE;
+		camel_sasl_set_authenticated (sasl, TRUE);
 		break;
 	default:
 		return NULL;
@@ -430,3 +423,32 @@ gssapi_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 }
 
 #endif /* HAVE_KRB5 */
+
+static void
+camel_sasl_gssapi_class_init (CamelSaslGssapiClass *class)
+{
+#ifdef HAVE_KRB5
+	GObjectClass *object_class;
+	CamelSaslClass *sasl_class;
+
+	g_type_class_add_private (class, sizeof (CamelSaslGssapiPrivate));
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->finalize = sasl_gssapi_finalize;
+
+	sasl_class = CAMEL_SASL_CLASS (class);
+	sasl_class->challenge_sync = sasl_gssapi_challenge_sync;
+#endif /* HAVE_KRB5 */
+}
+
+static void
+camel_sasl_gssapi_init (CamelSaslGssapi *sasl)
+{
+#ifdef HAVE_KRB5
+	sasl->priv = CAMEL_SASL_GSSAPI_GET_PRIVATE (sasl);
+
+	sasl->priv->state = GSSAPI_STATE_INIT;
+	sasl->priv->ctx = GSS_C_NO_CONTEXT;
+	sasl->priv->target = GSS_C_NO_NAME;
+#endif /* HAVE_KRB5 */
+}
