@@ -72,6 +72,7 @@ nntp_folder_finalize (GObject *object)
 gboolean
 camel_nntp_folder_selected (CamelNNTPFolder *nntp_folder,
                             gchar *line,
+                            GCancellable *cancellable,
                             GError **error)
 {
 	CamelFolder *folder;
@@ -83,11 +84,13 @@ camel_nntp_folder_selected (CamelNNTPFolder *nntp_folder,
 	return camel_nntp_summary_check (
 		CAMEL_NNTP_SUMMARY (folder->summary),
 		CAMEL_NNTP_STORE (parent_store),
-		line, nntp_folder->changes, error);
+		line, nntp_folder->changes,
+		cancellable, error);
 }
 
 static gboolean
 nntp_folder_refresh_info_online (CamelFolder *folder,
+                                 GCancellable *cancellable,
                                  GError **error)
 {
 	CamelStore *parent_store;
@@ -107,11 +110,11 @@ nntp_folder_refresh_info_online (CamelFolder *folder,
 	/* When invoked with no fmt, camel_nntp_command() just selects the folder
 	   and should return zero. */
 	success = !camel_nntp_command (
-		nntp_store, error, nntp_folder, &line, NULL);
+		nntp_store, cancellable, error, nntp_folder, &line, NULL);
 
-	if (camel_folder_change_info_changed(nntp_folder->changes)) {
+	if (camel_folder_change_info_changed (nntp_folder->changes)) {
 		changes = nntp_folder->changes;
-		nntp_folder->changes = camel_folder_change_info_new();
+		nntp_folder->changes = camel_folder_change_info_new ();
 	}
 
 	camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -188,8 +191,8 @@ nntp_get_filename (CamelFolder *folder, const gchar *uid, GError **error)
 	parent_store = camel_folder_get_parent_store (folder);
 	nntp_store = CAMEL_NNTP_STORE (parent_store);
 
-	article = alloca(strlen(uid)+1);
-	strcpy(article, uid);
+	article = alloca (strlen (uid)+1);
+	strcpy (article, uid);
 	msgid = strchr (article, ',');
 	if (msgid == NULL) {
 		g_set_error (
@@ -203,7 +206,11 @@ nntp_get_filename (CamelFolder *folder, const gchar *uid, GError **error)
 }
 
 static CamelStream *
-nntp_folder_download_message (CamelNNTPFolder *nntp_folder, const gchar *id, const gchar *msgid, GError **error)
+nntp_folder_download_message (CamelNNTPFolder *nntp_folder,
+                              const gchar *id,
+                              const gchar *msgid,
+                              GCancellable *cancellable,
+                              GError **error)
 {
 	CamelFolder *folder;
 	CamelStore *parent_store;
@@ -216,11 +223,11 @@ nntp_folder_download_message (CamelNNTPFolder *nntp_folder, const gchar *id, con
 	parent_store = camel_folder_get_parent_store (folder);
 	nntp_store = CAMEL_NNTP_STORE (parent_store);
 
-	ret = camel_nntp_command (nntp_store, error, nntp_folder, &line, "article %s", id);
+	ret = camel_nntp_command (nntp_store, cancellable, error, nntp_folder, &line, "article %s", id);
 	if (ret == 220) {
 		stream = camel_data_cache_add (nntp_store->cache, "cache", msgid, NULL);
 		if (stream) {
-			if (camel_stream_write_to_stream ((CamelStream *) nntp_store->stream, stream, error) == -1)
+			if (camel_stream_write_to_stream ((CamelStream *) nntp_store->stream, stream, cancellable, error) == -1)
 				goto fail;
 			if (camel_stream_reset (stream, error) == -1)
 				goto fail;
@@ -249,6 +256,7 @@ fail:
 static gboolean
 nntp_folder_cache_message (CamelDiscoFolder *disco_folder,
                            const gchar *uid,
+                           GCancellable *cancellable,
                            GError **error)
 {
 	CamelFolder *folder;
@@ -262,9 +270,9 @@ nntp_folder_cache_message (CamelDiscoFolder *disco_folder,
 	parent_store = camel_folder_get_parent_store (folder);
 	nntp_store = CAMEL_NNTP_STORE (parent_store);
 
-	article = alloca(strlen(uid)+1);
-	strcpy(article, uid);
-	msgid = strchr(article, ',');
+	article = alloca (strlen (uid)+1);
+	strcpy (article, uid);
+	msgid = strchr (article, ',');
 	if (!msgid) {
 		g_set_error (
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
@@ -276,7 +284,7 @@ nntp_folder_cache_message (CamelDiscoFolder *disco_folder,
 	camel_service_lock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	stream = nntp_folder_download_message (
-		(CamelNNTPFolder *) disco_folder, article, msgid, error);
+		(CamelNNTPFolder *) disco_folder, article, msgid, cancellable, error);
 	if (stream)
 		g_object_unref (stream);
 	else
@@ -287,8 +295,81 @@ nntp_folder_cache_message (CamelDiscoFolder *disco_folder,
 	return success;
 }
 
+static GPtrArray*
+nntp_folder_search_by_expression (CamelFolder *folder, const gchar *expression, GError **error)
+{
+	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
+	GPtrArray *matches;
+
+	CAMEL_NNTP_FOLDER_LOCK (nntp_folder, search_lock);
+
+	if (nntp_folder->search == NULL)
+		nntp_folder->search = camel_folder_search_new ();
+
+	camel_folder_search_set_folder (nntp_folder->search, folder);
+	matches = camel_folder_search_search (nntp_folder->search, expression, NULL, error);
+
+	CAMEL_NNTP_FOLDER_UNLOCK (nntp_folder, search_lock);
+
+	return matches;
+}
+
+static guint32
+nntp_folder_count_by_expression (CamelFolder *folder, const gchar *expression, GError **error)
+{
+	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
+	guint32 count;
+
+	CAMEL_NNTP_FOLDER_LOCK (nntp_folder, search_lock);
+
+	if (nntp_folder->search == NULL)
+		nntp_folder->search = camel_folder_search_new ();
+
+	camel_folder_search_set_folder (nntp_folder->search, folder);
+	count = camel_folder_search_count (nntp_folder->search, expression, error);
+
+	CAMEL_NNTP_FOLDER_UNLOCK (nntp_folder, search_lock);
+
+	return count;
+}
+
+static GPtrArray *
+nntp_folder_search_by_uids (CamelFolder *folder, const gchar *expression, GPtrArray *uids, GError **error)
+{
+	CamelNNTPFolder *nntp_folder = (CamelNNTPFolder *) folder;
+	GPtrArray *matches;
+
+	if (uids->len == 0)
+		return g_ptr_array_new ();
+
+	CAMEL_NNTP_FOLDER_LOCK (folder, search_lock);
+
+	if (nntp_folder->search == NULL)
+		nntp_folder->search = camel_folder_search_new ();
+
+	camel_folder_search_set_folder (nntp_folder->search, folder);
+	matches = camel_folder_search_search (nntp_folder->search, expression, uids, error);
+
+	CAMEL_NNTP_FOLDER_UNLOCK (folder, search_lock);
+
+	return matches;
+}
+
+static void
+nntp_folder_search_free (CamelFolder *folder, GPtrArray *result)
+{
+	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
+
+	CAMEL_NNTP_FOLDER_LOCK (nntp_folder, search_lock);
+	camel_folder_search_free_result (nntp_folder->search, result);
+	CAMEL_NNTP_FOLDER_UNLOCK (nntp_folder, search_lock);
+}
+
 static CamelMimeMessage *
-nntp_folder_get_message (CamelFolder *folder, const gchar *uid, GError **error)
+nntp_folder_get_message_sync (CamelFolder *folder,
+                              const gchar *uid,
+                              GCancellable *cancellable,
+                              GError **error)
 {
 	CamelStore *parent_store;
 	CamelMimeMessage *message = NULL;
@@ -303,8 +384,8 @@ nntp_folder_get_message (CamelFolder *folder, const gchar *uid, GError **error)
 	nntp_folder = CAMEL_NNTP_FOLDER (folder);
 	nntp_store = CAMEL_NNTP_STORE (parent_store);
 
-	article = alloca(strlen(uid)+1);
-	strcpy(article, uid);
+	article = alloca (strlen (uid)+1);
+	strcpy (article, uid);
 	msgid = strchr (article, ',');
 	if (msgid == NULL) {
 		g_set_error (
@@ -327,13 +408,13 @@ nntp_folder_get_message (CamelFolder *folder, const gchar *uid, GError **error)
 			goto fail;
 		}
 
-		stream = nntp_folder_download_message (nntp_folder, article, msgid, error);
+		stream = nntp_folder_download_message (nntp_folder, article, msgid, cancellable, error);
 		if (stream == NULL)
 			goto fail;
 	}
 
 	message = camel_mime_message_new ();
-	if (camel_data_wrapper_construct_from_stream ((CamelDataWrapper *) message, stream, error) == -1) {
+	if (!camel_data_wrapper_construct_from_stream_sync ((CamelDataWrapper *) message, stream, cancellable, error)) {
 		g_prefix_error (error, _("Cannot get message %s: "), uid);
 		g_object_unref (message);
 		message = NULL;
@@ -358,81 +439,12 @@ fail:
 	return message;
 }
 
-static GPtrArray*
-nntp_folder_search_by_expression (CamelFolder *folder, const gchar *expression, GError **error)
-{
-	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
-	GPtrArray *matches;
-
-	CAMEL_NNTP_FOLDER_LOCK(nntp_folder, search_lock);
-
-	if (nntp_folder->search == NULL)
-		nntp_folder->search = camel_folder_search_new ();
-
-	camel_folder_search_set_folder (nntp_folder->search, folder);
-	matches = camel_folder_search_search(nntp_folder->search, expression, NULL, error);
-
-	CAMEL_NNTP_FOLDER_UNLOCK(nntp_folder, search_lock);
-
-	return matches;
-}
-
-static guint32
-nntp_folder_count_by_expression (CamelFolder *folder, const gchar *expression, GError **error)
-{
-	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
-	guint32 count;
-
-	CAMEL_NNTP_FOLDER_LOCK(nntp_folder, search_lock);
-
-	if (nntp_folder->search == NULL)
-		nntp_folder->search = camel_folder_search_new ();
-
-	camel_folder_search_set_folder (nntp_folder->search, folder);
-	count = camel_folder_search_count(nntp_folder->search, expression, error);
-
-	CAMEL_NNTP_FOLDER_UNLOCK(nntp_folder, search_lock);
-
-	return count;
-}
-
-static GPtrArray *
-nntp_folder_search_by_uids (CamelFolder *folder, const gchar *expression, GPtrArray *uids, GError **error)
-{
-	CamelNNTPFolder *nntp_folder = (CamelNNTPFolder *) folder;
-	GPtrArray *matches;
-
-	if (uids->len == 0)
-		return g_ptr_array_new();
-
-	CAMEL_NNTP_FOLDER_LOCK(folder, search_lock);
-
-	if (nntp_folder->search == NULL)
-		nntp_folder->search = camel_folder_search_new ();
-
-	camel_folder_search_set_folder (nntp_folder->search, folder);
-	matches = camel_folder_search_search(nntp_folder->search, expression, uids, error);
-
-	CAMEL_NNTP_FOLDER_UNLOCK(folder, search_lock);
-
-	return matches;
-}
-
-static void
-nntp_folder_search_free (CamelFolder *folder, GPtrArray *result)
-{
-	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
-
-	CAMEL_NNTP_FOLDER_LOCK(nntp_folder, search_lock);
-	camel_folder_search_free_result (nntp_folder->search, result);
-	CAMEL_NNTP_FOLDER_UNLOCK(nntp_folder, search_lock);
-}
-
 static gboolean
 nntp_folder_append_message_online (CamelFolder *folder,
                                    CamelMimeMessage *mime_message,
                                    const CamelMessageInfo *info,
                                    gchar **appended_uid,
+                                   GCancellable *cancellable,
                                    GError **error)
 {
 	CamelStore *parent_store;
@@ -456,7 +468,7 @@ nntp_folder_append_message_online (CamelFolder *folder,
 	camel_service_lock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	/* send 'POST' command */
-	ret = camel_nntp_command (nntp_store, error, NULL, &line, "post");
+	ret = camel_nntp_command (nntp_store, cancellable, error, NULL, &line, "post");
 	if (ret != 340) {
 		if (ret == 440) {
 			g_set_error (
@@ -505,14 +517,14 @@ nntp_folder_append_message_online (CamelFolder *folder,
 	}
 
 	/* write the message */
-	if (camel_stream_write(stream, group, strlen(group), error) == -1
-	    || camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (mime_message), filtered_stream, error) == -1
-	    || camel_stream_flush (filtered_stream, error) == -1
-	    || camel_stream_write (stream, "\r\n.\r\n", 5, error) == -1
-	    || (ret = camel_nntp_stream_line (nntp_store->stream, (guchar **)&line, &u, error)) == -1) {
+	if (camel_stream_write (stream, group, strlen (group), cancellable, error) == -1
+	    || camel_data_wrapper_write_to_stream_sync (CAMEL_DATA_WRAPPER (mime_message), filtered_stream, cancellable, error) == -1
+	    || camel_stream_flush (filtered_stream, cancellable, error) == -1
+	    || camel_stream_write (stream, "\r\n.\r\n", 5, cancellable, error) == -1
+	    || camel_nntp_stream_line (nntp_store->stream, (guchar **)&line, &u, cancellable, error) == -1) {
 		g_prefix_error (error, _("Posting failed: "));
 		success = FALSE;
-	} else if (atoi(line) != 240) {
+	} else if (atoi (line) != 240) {
 		g_set_error (
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Posting failed: %s"), line);
@@ -520,7 +532,7 @@ nntp_folder_append_message_online (CamelFolder *folder,
 	}
 
 	g_object_unref (filtered_stream);
-	g_free(group);
+	g_free (group);
 	header->next = savedhdrs;
 
 	camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -533,6 +545,7 @@ nntp_folder_append_message_offline (CamelFolder *folder,
                                     CamelMimeMessage *mime_message,
                                     const CamelMessageInfo *info,
                                     gchar **appended_uid,
+                                    GCancellable *cancellable,
                                     GError **error)
 {
 	g_set_error (
@@ -552,6 +565,7 @@ nntp_folder_transfer_message (CamelFolder *source,
                               CamelFolder *dest,
                               GPtrArray **transferred_uids,
                               gboolean delete_orig,
+                              GCancellable *cancellable,
                               GError **error)
 {
 	g_set_error (
@@ -602,12 +616,12 @@ camel_nntp_folder_class_init (CamelNNTPFolderClass *class)
 	object_class->finalize = nntp_folder_finalize;
 
 	folder_class = CAMEL_FOLDER_CLASS (class);
-	folder_class->get_message = nntp_folder_get_message;
 	folder_class->search_by_expression = nntp_folder_search_by_expression;
 	folder_class->count_by_expression = nntp_folder_count_by_expression;
 	folder_class->search_by_uids = nntp_folder_search_by_uids;
 	folder_class->search_free = nntp_folder_search_free;
 	folder_class->get_filename = nntp_get_filename;
+	folder_class->get_message_sync = nntp_folder_get_message_sync;
 
 	disco_folder_class = CAMEL_DISCO_FOLDER_CLASS (class);
 	disco_folder_class->sync_online = nntp_folder_sync_online;
@@ -639,6 +653,7 @@ camel_nntp_folder_init (CamelNNTPFolder *nntp_folder)
 CamelFolder *
 camel_nntp_folder_new (CamelStore *parent,
                        const gchar *folder_name,
+                       GCancellable *cancellable,
                        GError **error)
 {
 	CamelFolder *folder;
@@ -671,11 +686,11 @@ camel_nntp_folder_new (CamelStore *parent,
 	root = g_strdup_printf ("%s.cmeta", nntp_folder->storage_path);
 	camel_object_set_state_filename (CAMEL_OBJECT (nntp_folder), root);
 	camel_object_state_read (CAMEL_OBJECT (nntp_folder));
-	g_free(root);
+	g_free (root);
 
 	root = g_strdup_printf("%s.ev-summary", nntp_folder->storage_path);
 	folder->summary = (CamelFolderSummary *) camel_nntp_summary_new (folder, root);
-	g_free(root);
+	g_free (root);
 
 	camel_folder_summary_load_from_db (folder->summary, NULL);
 
@@ -685,10 +700,11 @@ camel_nntp_folder_new (CamelStore *parent,
 		camel_store_summary_info_free ((CamelStoreSummary *) ((CamelNNTPStore*) parent)->summary, si);
 	}
 
-	if (subscribed && !camel_folder_refresh_info (folder, error)) {
+	if (subscribed && !camel_folder_refresh_info_sync (
+			folder, cancellable, error)) {
 		g_object_unref (folder);
 		folder = NULL;
-        }
+	}
 
 	return folder;
 }
