@@ -35,17 +35,10 @@
 #include "e-name-selector-dialog.h"
 #include "e-name-selector-entry.h"
 
-/* backward-compatibility cruft */
-#include "gtk-compat.h"
-
 #define E_NAME_SELECTOR_DIALOG_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE \
 	((obj), E_TYPE_NAME_SELECTOR_DIALOG, ENameSelectorDialogPrivate))
 
-#if !GTK_CHECK_VERSION (2,23,0)
-	ENSURE_GTK_COMBO_BOX_TEXT_TYPE
-#endif
-
 typedef struct {
 	gchar        *name;
 
@@ -408,9 +401,6 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	gtk_window_set_modal            (GTK_WINDOW (name_selector_dialog), TRUE);
 	gtk_window_set_default_size     (GTK_WINDOW (name_selector_dialog), 700, -1);
 	gtk_window_set_resizable        (GTK_WINDOW (name_selector_dialog), TRUE);
-#if !GTK_CHECK_VERSION(2,90,7)
-	g_object_set (name_selector_dialog, "has-separator", FALSE, NULL);
-#endif
 	gtk_container_set_border_width  (GTK_CONTAINER (name_selector_dialog), 4);
 	gtk_window_set_title            (GTK_WINDOW (name_selector_dialog), _("Select Contacts from Address Book"));
 	gtk_widget_grab_focus (search);
